@@ -76,6 +76,10 @@ print_usage(void)
 	MSG("  # makedumpfile --split [OPTION] [-x VMLINUX|-i VMCOREINFO] VMCORE DUMPFILE1\n");
 	MSG("    DUMPFILE2 [DUMPFILE3 ..]\n");
 	MSG("\n");
+	MSG("  Using multiple threads to create DUMPFILE in parallel:\n");
+	MSG("  # makedumpfile [OPTION] [-x VMLINUX|-i VMCOREINFO] --num-threads THREADNUM\n");
+	MSG("    VMCORE DUMPFILE1\n");
+	MSG("\n");
 	MSG("  Reassemble multiple DUMPFILEs:\n");
 	MSG("  # makedumpfile --reassemble DUMPFILE1 DUMPFILE2 [DUMPFILE3 ..] DUMPFILE\n");
 	MSG("\n");
@@ -87,10 +91,11 @@ print_usage(void)
 	MSG("\n");
 	MSG("\n");
 	MSG("  Creating DUMPFILE of Xen:\n");
-	MSG("  # makedumpfile -E [--xen-syms XEN-SYMS|--xen-vmcoreinfo VMCOREINFO] VMCORE DUMPFILE\n");
+	MSG("  # makedumpfile [-c|-l|-p|-E] [--xen-syms XEN-SYMS|--xen-vmcoreinfo VMCOREINFO]\n");
+	MSG("    VMCORE DUMPFILE\n");
 	MSG("\n");
 	MSG("  Filtering domain-0 of Xen:\n");
-	MSG("  # makedumpfile -E -d DL -x vmlinux VMCORE DUMPFILE\n");
+	MSG("  # makedumpfile [-c|-l|-p|-E] -d DL -x vmlinux VMCORE DUMPFILE\n");
 	MSG("\n");
 	MSG("  Generating VMCOREINFO of Xen:\n");
 	MSG("  # makedumpfile -g VMCOREINFO --xen-syms XEN-SYMS\n");
@@ -184,6 +189,12 @@ print_usage(void)
 	MSG("      by the number of DUMPFILEs.\n");
 	MSG("      This feature supports only the kdump-compressed format.\n");
 	MSG("\n");
+	MSG("  [--num-threads THREADNUM]:\n");
+	MSG("      Using multiple threads to read and compress data of each page in parallel.\n");
+	MSG("      And it will reduces time for saving DUMPFILE.\n");
+	MSG("      This feature only supports creating DUMPFILE in kdump-comressed format from\n");
+	MSG("      VMCORE in kdump-compressed format or elf format.\n");
+	MSG("\n");
 	MSG("  [--reassemble]:\n");
 	MSG("      Reassemble multiple DUMPFILEs, which are created by --split option,\n");
 	MSG("      into one DUMPFILE. dumpfile1 and dumpfile2 are reassembled into dumpfile.\n");
@@ -193,8 +204,9 @@ print_usage(void)
 	MSG("      writing to output. The default value is 4.\n");
 	MSG("\n");
 	MSG("  [--cyclic-buffer BUFFER_SIZE]:\n");
-	MSG("      Specify the buffer size in kilo bytes for analysis in the cyclic mode.\n");
-	MSG("      In the cyclic mode, the number of cycles is represented as:\n");
+	MSG("      Specify the buffer size in kilo bytes for bitmap data.\n");
+	MSG("      Filtering processing will be divided into multi cycles to fix the memory\n");
+	MSG("      consumption, the number of cycles is represented as:\n");
 	MSG("\n");
 	MSG("          num_of_cycles = system_memory / \n");
 	MSG("                          (BUFFER_SIZE * 1024 * bit_per_bytes * page_size)\n");
@@ -204,14 +216,16 @@ print_usage(void)
 	MSG("      system memory size, so ordinary users don't need to specify this option.\n");
 	MSG("\n");
 	MSG("  [--splitblock-size SPLITBLOCK_SIZE]:\n");
-	MSG("      Specify the splitblock size in kilo bytes for analysis in the cyclic mode\n");
-	MSG("      with --split. If --splitblock N is specified, difference of each splitted\n");
-	MSG("      dumpfile size is at most N kilo bytes.\n");
+	MSG("      Specify the splitblock size in kilo bytes for analysis with --split.\n");
+	MSG("      If --splitblock N is specified, difference of each splitted dumpfile\n");
+	MSG("      size is at most N kilo bytes.\n");
 	MSG("\n");
-	MSG("  [--non-cyclic]:\n");
-	MSG("      Running in the non-cyclic mode, this mode uses the old filtering logic\n");
-	MSG("      same as v1.4.4 or before.\n");
-	MSG("      If you feel the cyclic mode is too slow, please try this mode.\n");
+	MSG("  [--work-dir]:\n");
+	MSG("      Specify the working directory for the temporary bitmap file.\n");
+	MSG("      If this option isn't specified, the bitmap will be saved on memory.\n");
+	MSG("      Filtering processing has to do 2 pass scanning to fix the memory consumption,\n");
+	MSG("      but it can be avoided by using working directory on file system.\n");
+	MSG("      So if you specify this option, the filtering speed may be bit faster.\n");
 	MSG("\n");
 	MSG("  [--non-mmap]:\n");
 	MSG("      Never use mmap(2) to read VMCORE even if it supports mmap(2).\n");
